@@ -30,6 +30,7 @@
 #include "panoinc.h"
 
 #include <panotools/PanoToolsOptimizerWrapper.h>
+#include "hugin_base/panotools/PanoToolsUtils.h"
 #include <algorithms/optimizer/PTOptimizer.h>
 #include <algorithms/basic/CalculateCPStatistics.h>
 
@@ -43,19 +44,12 @@
 #include "panodata/OptimizerSwitches.h"
 #include "hugin/PanoOperation.h"
 #include "base_wx/LensTools.h"
+#include "base_wx/wxutils.h"
 
 //============================================================================
 //============================================================================
 //============================================================================
 
-BEGIN_EVENT_TABLE(OptimizePanel, wxPanel)
-    EVT_CLOSE(OptimizePanel::OnClose)
-    EVT_BUTTON(XRCID("optimize_panel_optimize"), OptimizePanel::OnOptimizeButton)
-    EVT_BUTTON(XRCID("optimize_panel_reset"), OptimizePanel::OnReset)
-    EVT_CHECKBOX(XRCID("optimizer_panel_only_active_images"), OptimizePanel::OnCheckOnlyActiveImages)
-END_EVENT_TABLE()
-
-
 OptimizePanel::OptimizePanel()
 {
     DEBUG_TRACE("");
@@ -72,7 +66,7 @@ bool OptimizePanel::Create(wxWindow* parent, wxWindowID id , const wxPoint& pos,
     // create a sub-panel and load class into it!
 
     // wxPanel::Create is called in here!
-    wxXmlResource::Get()->LoadPanel(this, wxT("optimize_panel"));
+    wxXmlResource::Get()->LoadPanel(this, "optimize_panel");
     wxPanel * panel = XRCCTRL(*this, "optimize_panel", wxPanel);
 
     wxBoxSizer *topsizer = new wxBoxSizer( wxVERTICAL );
@@ -81,18 +75,27 @@ bool OptimizePanel::Create(wxWindow* parent, wxWindowID id , const wxPoint& pos,
 
     m_only_active_images_cb = XRCCTRL(*this, "optimizer_panel_only_active_images", wxCheckBox);
     DEBUG_ASSERT(m_only_active_images_cb);
-    m_only_active_images_cb->SetValue(wxConfigBase::Get()->Read(wxT("/OptimizePanel/OnlyActiveImages"),1l) != 0);
-    MainFrame::Get()->SetOptimizeOnlyActiveImages(m_only_active_images_cb->GetValue());
+    m_only_active_images_cb->SetValue(false);
+    m_only_active_images_cb->Bind(wxEVT_CHECKBOX, &OptimizePanel::OnCheckOnlyActiveImages, this);
+    m_ignore_line_cp = XRCCTRL(*this, "optimizer_panel_ignore_line_cp", wxCheckBox);
+    DEBUG_ASSERT(m_ignore_line_cp);
+    m_ignore_line_cp->SetValue(false);
+    m_ignore_line_cp->Bind(wxEVT_CHECKBOX, &OptimizePanel::OnCheckIgnoreLineCP, this);
 
     m_images_tree_list = XRCCTRL(*this, "optimize_panel_images", ImagesTreeCtrl);
     DEBUG_ASSERT(m_images_tree_list);
     m_lens_tree_list = XRCCTRL(*this, "optimize_panel_lenses", ImagesTreeCtrl);
     DEBUG_ASSERT(m_lens_tree_list);
+    SetOnlyActiveImages(wxConfigBase::Get()->Read("/OptimizePanel/OnlyActiveImages", 1l) != 0);
 
     m_edit_cb = XRCCTRL(*this, "optimizer_panel_edit_script", wxCheckBox);
     DEBUG_ASSERT(m_edit_cb);
 
     XRCCTRL(*this, "optimizer_panel_splitter", wxSplitterWindow)->SetSashGravity(0.66);
+    // bind other events
+    Bind(wxEVT_CLOSE_WINDOW, &OptimizePanel::OnClose, this);
+    Bind(wxEVT_BUTTON, &OptimizePanel::OnOptimizeButton, this, XRCID("optimize_panel_optimize"));
+    Bind(wxEVT_BUTTON, &OptimizePanel::OnReset, this, XRCID("optimize_panel_reset"));
 
     return true;
 }
@@ -122,9 +125,6 @@ void OptimizePanel::SetGuiLevel(GuiLevel newGuiLevel)
 
 OptimizePanel::~OptimizePanel()
 {
-    DEBUG_TRACE("dtor, writing config");
-    wxConfigBase::Get()->Write(wxT("/OptimizePanel/OnlyActiveImages"),m_only_active_images_cb->IsChecked() ? 1l : 0l);
-
     m_pano->removeObserver(this);
     DEBUG_TRACE("dtor end");
 }
@@ -148,21 +148,15 @@ void OptimizePanel::OnOptimizeButton(wxCommandEvent & e)
 {
     DEBUG_TRACE("");
     // run optimizer
-
     HuginBase::UIntSet imgs;
     if (m_only_active_images_cb->IsChecked() || m_pano->getOptimizerSwitch()!=0)
     {
         // use only selected images.
         imgs = m_pano->getActiveImages();
-        if (imgs.size() == 0)
+        if (imgs.empty())
         {
-            wxMessageBox(_("The project does not contain any active images.\nPlease activate at least one image in the (fast) preview window.\nOptimization canceled."),
-#ifdef _WIN32
-                _("Hugin"),
-#else
-                wxT(""),
-#endif
-                wxICON_ERROR | wxOK);
+            hugin_utils::HuginMessageBox(_("The project does not contain any active images.\nPlease activate at least one image in the (fast) preview window.\nOptimization canceled."),
+                _("Hugin"), wxICON_ERROR | wxOK, wxGetActiveWindow());
             return;
         }
     }
@@ -172,20 +166,27 @@ void OptimizePanel::OnOptimizeButton(wxCommandEvent & e)
     }
     if (CheckLensStacks(m_pano, true))
     {
-        runOptimizer(imgs);
+        runOptimizer(imgs, m_ignore_line_cp->IsChecked());
     };
 }
 
-void OptimizePanel::runOptimizer(const HuginBase::UIntSet & imgs)
+void OptimizePanel::runOptimizer(const HuginBase::UIntSet & imgs, const bool ignoreLineCp)
 {
     DEBUG_TRACE("");
     // open window that shows a status dialog, and allows to
     // apply the results
-    int mode=m_pano->getOptimizerSwitch();
-    // remember active window for dialogs
-    wxWindow* activeWindow = wxGetActiveWindow();
+    const int mode=m_pano->getOptimizerSwitch();
 
     HuginBase::Panorama optPano = m_pano->getSubset(imgs);
+    if (optPano.getNrOfCtrlPoints() == 0)
+    {
+        hugin_utils::HuginMessageBox(_("There are no control points in the current configuration for the optimizer.\nPlease add control points before running the optimizer.\nOptimization canceled."),
+            _("Hugin"), wxICON_ERROR | wxOK, wxGetActiveWindow());
+        return;
+    };
+    // disable optimize button, so user can't click twice
+    hugin_utils::DisableWindow disableButton(XRCCTRL(*this, "optimize_panel_optimize", wxButton));
+
     HuginBase::PanoramaOptions opts = optPano.getOptions();
     switch(opts.getProjection())
     {
@@ -205,6 +206,7 @@ void OptimizePanel::runOptimizer(const HuginBase::UIntSet & imgs)
     char *p = setlocale(LC_ALL,NULL);
     char *oldlocale = strdup(p);
     setlocale(LC_ALL,"C");
+    HuginBase::CPVector originalCps;
 
     if (mode & HuginBase::OPT_PAIR )
     {
@@ -233,18 +235,40 @@ void OptimizePanel::runOptimizer(const HuginBase::UIntSet & imgs)
     }
     else
     {
+        HuginBase::CPVector optCps;
+        if (ignoreLineCp)
+        {
+            // store for later
+            originalCps = optPano.getCtrlPoints();
+            // remove all line cp
+            for (auto& cp : originalCps)
+            {
+                if (cp.mode == HuginBase::ControlPoint::X_Y)
+                {
+                    optCps.push_back(cp);
+                };
+            };
+            if (optCps.empty())
+            {
+                hugin_utils::HuginMessageBox(_("There are no control points in the current configuration for the optimizer.\nPlease add control points before running the optimizer.\nOptimization canceled."),
+                    _("Hugin"), wxICON_ERROR | wxOK, wxGetActiveWindow());
+                return;
+            }
+            optPano.setCtrlPoints(optCps);
+        };
         if (m_edit_cb->IsChecked() && mode==0)
         {
             // show and edit script..
             std::ostringstream scriptbuf;
             optPano.printPanoramaScript(scriptbuf, optPano.getOptimizeVector(), optPano.getOptions(), allImg, true);
             // open a text dialog with an editor inside
-            wxDialog * edit_dlg = wxXmlResource::Get()->LoadDialog(this, wxT("edit_script_dialog"));
-            wxTextCtrl *txtCtrl=XRCCTRL(*edit_dlg,"script_edit_text",wxTextCtrl);
+            wxDialog edit_dlg;
+            wxXmlResource::Get()->LoadDialog(&edit_dlg, this, "edit_script_dialog");
+            wxTextCtrl *txtCtrl=XRCCTRL(edit_dlg,"script_edit_text",wxTextCtrl);
             txtCtrl->SetValue(wxString(scriptbuf.str().c_str(), *wxConvCurrent));
 
             char * script = 0;
-            if (edit_dlg->ShowModal() == wxID_OK)
+            if (edit_dlg.ShowModal() == wxID_OK)
             {
                 script = strdup(txtCtrl->GetValue().mb_str(*wxConvCurrent));
             }
@@ -277,8 +301,15 @@ void OptimizePanel::runOptimizer(const HuginBase::UIntSet & imgs)
     wxTheApp->ProcessIdle();
 #endif
     // calculate control point errors and display text.
-    if (AskApplyResult(activeWindow, optPano))
+    if (AskApplyResult(wxGetActiveWindow(), optPano))
     {
+        if (!originalCps.empty())
+        {
+            // restore all control points
+            optPano.setCtrlPoints(originalCps);
+            // because the line cp were removed we need to calculate the cp error again
+            HuginBase::PTools::calcCtrlPointErrors(optPano);
+        };
         PanoCommand::GlobalCmdHist::getInstance().addCommand(
             new PanoCommand::UpdateVariablesCPSetCmd(*m_pano, imgs, optPano.getVariables(), optPano.getCtrlPoints())
         );
@@ -291,7 +322,7 @@ bool OptimizePanel::AskApplyResult(wxWindow* activeWindow, const HuginBase::Pano
     double max;
     double mean;
     double var;
-    HuginBase::CalculateCPStatisticsError::calcCtrlPntsErrorStats(pano, min, max, mean, var, -1, m_only_active_images_cb->IsChecked());
+    HuginBase::CalculateCPStatisticsError::calcCtrlPntsErrorStats(pano, min, max, mean, var, -1);
 
     // check for HFOV lines. if smaller than 1 report a warning;
     // also check for high distortion coefficients.
@@ -329,9 +360,7 @@ bool OptimizePanel::AskApplyResult(wxWindow* activeWindow, const HuginBase::Pano
         }
     };
 
-    int id = wxMessageBox(msg, _("Optimization result"), style, activeWindow);
-
-    return id == wxYES;
+    return hugin_utils::HuginMessageBox(msg, _("Hugin"), style, activeWindow) == wxYES;
 }
 
 void OptimizePanel::OnClose(wxCloseEvent& event)
@@ -370,6 +399,18 @@ void OptimizePanel::OnCheckOnlyActiveImages(wxCommandEvent &e)
 void OptimizePanel::SetOnlyActiveImages(const bool onlyActive)
 {
     m_only_active_images_cb->SetValue(onlyActive);
+    m_images_tree_list->MarkActiveImages(onlyActive);
+    m_lens_tree_list->MarkActiveImages(onlyActive);
+};
+
+void OptimizePanel::OnCheckIgnoreLineCP(wxCommandEvent &e)
+{
+    MainFrame::Get()->SetOptimizeIgnoreLineCp(m_ignore_line_cp->IsChecked());
+};
+
+void OptimizePanel::SetIgnoreLineCP(const bool noLineCp)
+{
+    m_ignore_line_cp->SetValue(noLineCp);
 };
 
 IMPLEMENT_DYNAMIC_CLASS(OptimizePanel, wxPanel)
@@ -388,7 +429,7 @@ wxObject *OptimizePanelXmlHandler::DoCreateResource()
     cp->Create(m_parentAsWindow,
                    GetID(),
                    GetPosition(), GetSize(),
-                   GetStyle(wxT("style")),
+                   GetStyle("style"),
                    GetName());
 
     SetupWindow( cp);
@@ -398,7 +439,7 @@ wxObject *OptimizePanelXmlHandler::DoCreateResource()
 
 bool OptimizePanelXmlHandler::CanHandle(wxXmlNode *node)
 {
-    return IsOfClass(node, wxT("OptimizePanel"));
+    return IsOfClass(node, "OptimizePanel");
 }
 
 IMPLEMENT_DYNAMIC_CLASS(OptimizePanelXmlHandler, wxXmlResourceHandler)
