@@ -28,11 +28,7 @@
 
 #include <iostream>
 #include "hugin_config.h"
-#ifdef USE_CXX11_THREAD
 #include <thread>
-#else
-#include <boost/thread/thread.hpp>
-#endif
 #include <vigra/inspectimage.hxx>
 #include <vigra/accessor.hxx>
 #include <vigra/functorexpression.hxx>
@@ -42,44 +38,10 @@
 #include <vigra_ext/Pyramid.h>
 #include <vigra_ext/FunctorAccessor.h>
 
-
-
 namespace HuginBase {
-    
-template <class T1>
-class GetRange
-{
-    public:
-        static T1 min();
-        static T1 max();
-};
-
-// ImageCache::GetRange implementation
-#define VIGRA_EXT_GETRANGE(T1, MI,MA) \
-template<> \
-T1 GetRange<T1>::min() \
-{ \
-        return MI; \
-} \
-template<> \
-T1 GetRange<T1>::max() \
-{ \
-        return MA; \
-} \
-
-VIGRA_EXT_GETRANGE(vigra::UInt8,  0, 255);
-VIGRA_EXT_GETRANGE(vigra::Int16,  0, 32767);
-VIGRA_EXT_GETRANGE(vigra::UInt16, 0, 65535);
-VIGRA_EXT_GETRANGE(vigra::Int32,  0, 2147483647);
-VIGRA_EXT_GETRANGE(vigra::UInt32, 0, 4294967295u);
-VIGRA_EXT_GETRANGE(float,  0, 1.0f);
-VIGRA_EXT_GETRANGE(double, 0, 1.0);
-
-#undef VIGRA_EXT_GETRANGE
-
 
 template <class SrcIMG>
-void convertTo8Bit(SrcIMG & src, const std::string & origType, vigra::BRGBImage & dest)
+void convertTo8Bit(SrcIMG& src, const std::string& origType, vigra::BRGBImage& dest, int desiredMapping = -1)
 {
     // code to apply the mapping to 8 bit
     // always scaled from 0..1 for integer images.
@@ -88,27 +50,25 @@ void convertTo8Bit(SrcIMG & src, const std::string & origType, vigra::BRGBImage
     
     double min=0;
     double max=vigra_ext::getMaxValForPixelType(origType);
-    ;
     
     int mapping = HUGIN_IMGCACHE_MAPPING_INTEGER;
     
     // float needs to be from min ... max.
-    if (origType == "FLOAT" || origType == "DOUBLE")
+    if (origType == "FLOAT" || origType == "DOUBLE" || origType == "INT16" || origType == "UINT32" || origType == "INT32")
     {
         vigra::RGBToGrayAccessor<vigra::RGBValue<float> > ga;
-        vigra::FindMinMax<float> minmax;   // init functor
-        vigra::inspectImage(srcImageRange(src, ga),
-                            minmax);
-        min = minmax.min;
-        max = minmax.max;
-        mapping = HUGIN_IMGCACHE_MAPPING_FLOAT;
+        vigra::FindAverageAndVariance<float> mean;   // init functor
+        vigra::inspectImage(srcImageRange(src, ga), mean);
+        min = std::max(mean.average() - 3 * sqrt(mean.variance()), 1e-6f);
+        max = mean.average() + 3 * sqrt(mean.variance());
+        mapping = desiredMapping != -1 ? desiredMapping : HUGIN_IMGCACHE_MAPPING_FLOAT;
     }
     vigra_ext::applyMapping(srcImageRange(src), destImage(dest), min, max, mapping);
 }
 
 
 
-ImageCache::ImageCacheRGB8Ptr ImageCache::Entry::get8BitImage()
+ImageCache::ImageCacheRGB8Ptr ImageCache::Entry::get8BitImage(int desiredMapping)
 {
     if (image8->width() > 0) {
         return image8;
@@ -119,7 +79,8 @@ ImageCache::ImageCacheRGB8Ptr ImageCache::Entry::get8BitImage()
     } else if (imageFloat->width() > 0) {
         convertTo8Bit(*imageFloat,
                       origType,
-                      *image8);
+                      *image8, 
+                      desiredMapping);
     }
     return image8;
 }
@@ -134,7 +95,7 @@ void ImageCache::removeImage(const std::string & filename)
         images.erase(it);
     }
 
-    std::string sfilename = filename + std::string("_small");
+    std::string sfilename = filename + std::string(":small");
     it = images.find(sfilename);
     if (it != images.end()) {
         images.erase(it);
@@ -177,12 +138,14 @@ void ImageCache::flush()
 
 void ImageCache::softFlush()
 {
-    if(upperBound==0l)
-        upperBound = 100 * 1024 * 1024l;
-    long purgeToSize = long(0.75 * upperBound);
+    if (upperBound == 0ull)
+    {
+        upperBound = 100 * 1024 * 1024ull;
+    };
+    const unsigned long long purgeToSize = static_cast<unsigned long long>(0.75 * upperBound);
 
     // calculate used memory
-    long imgMem = 0;
+    unsigned long long imgMem = 0;
 
     std::map<std::string, EntryPtr>::iterator imgIt;
     for(imgIt=images.begin(); imgIt != images.end(); ++imgIt) {
@@ -216,20 +179,20 @@ void ImageCache::softFlush()
         }
     }
 
-    long pyrMem = 0;
+    unsigned long long pyrMem = 0;
     std::map<std::string, vigra::BImage*>::iterator pyrIt;
     for(pyrIt=pyrImages.begin(); pyrIt != pyrImages.end(); ++pyrIt) {
         pyrMem += pyrIt->second->width() * pyrIt->second->height();
     }
 
-    long usedMem = imgMem + pyrMem;
+    const unsigned long long usedMem = imgMem + pyrMem;
 
     DEBUG_DEBUG("total: " << (usedMem>>20) << " MB upper bound: " << (purgeToSize>>20) << " MB");
     if (usedMem > upperBound) 
     {
         // we need to remove images.
-        long purgeAmount = usedMem - purgeToSize;
-        long purgedMem = 0;
+        const unsigned long long purgeAmount = usedMem - purgeToSize;
+        unsigned long long purgedMem = 0;
         // remove images from cache, first the grey level image,
         // then the full size images
 
@@ -292,8 +255,6 @@ void ImageCache::softFlush()
     }
 }
 
-
-
 ImageCache& ImageCache::getInstance()
 {
     if (instance == NULL)
@@ -303,90 +264,12 @@ ImageCache& ImageCache::getInstance()
     return *instance;
 }
 
-
-
-/*
-struct ApplyGammaFunctor
-{
-    float minv;
-    float maxv;
-    float gamma;
-    float scale;
-
-    ApplyGammaFunctor(float min_, float max_, float gamma_)
-    {
-        minv = min_;
-        maxv = max_;
-        gamma = gamma_;
-        scale = maxv - minv;
-    }
-
-    template <class T>
-    unsigned char operator()(T v) const
-    {
-        typedef vigra::NumericTraits<vigra::UInt8>  DestTraits;
-        return DestTraits::fromRealPromote(pow((float(v)-minv)/scale, gamma)*255);
-    }
-
-    template <class T, unsigned int R, unsigned int G, unsigned int B>
-    RGBValue<vigra::UInt8,0,1,2> operator()(const RGBValue<T,R,G,B> & v) const
-    {
-        typedef vigra::NumericTraits< RGBValue<vigra::UInt8,0,1,2> >  DestTraits;
-        typedef vigra::NumericTraits< RGBValue<T,R,G,B> >  SrcTraits;
-        return DestTraits::fromRealPromote(pow((SrcTraits::toRealPromote(v)+(-minv))/scale, gamma)*255);
-//        return DestTraits::fromRealPromote((log10(SrcTraits::toRealPromote(v)) + (-minv))/scale);
-    }
-};
-*/
-
-
-//#if 0
-///// add a scalar to all components, might break other stuff, therefore define just here
-//template <class V1, class V2>
-//inline
-//vigra::RGBValue<V1>
-//operator+(const vigra::RGBValue<V1> l, V2 const & r)
-//{
-//    return vigra::RGBValue<V1>(l.red() + r, l.green() + r, l.blue() + r);
-//}
-//
-///// subtract a scalar to all components, might break other stuff, therefore define just here
-//template <class V1, class V2>
-//inline
-//vigra::RGBValue<V1>
-//operator-(const vigra::RGBValue<V1> l, V2 const & r)
-//{
-//    return vigra::RGBValue<V1>(l.red() - r, l.green() - r, l.blue() - r);
-//}
-//#endif
-
-
-
-
-//struct MyMultFunc
-//{
-//    MyMultFunc(double f)
-//    {
-//        m = f;
-//    }
-//    double m;
-//
-//    template<class T>
-//    T
-//    operator()(T v) const
-//    {
-//        return vigra::NumericTraits<T>::fromRealPromote(v*m);
-//    }
-//};
-
 template <class SrcPixelType,
           class DestIterator, class DestAccessor>
 void ImageCache::importAndConvertImage(const vigra::ImageImportInfo & info,
                                        vigra::pair<DestIterator, DestAccessor> dest,
                                        const std::string & type)
 {
-    typedef typename DestAccessor::value_type DestPixelType;
-
     if (type == "FLOAT" || type == "DOUBLE" ) {
         // import image as it is
         vigra::importImage(info, dest);
@@ -394,94 +277,12 @@ void ImageCache::importAndConvertImage(const vigra::ImageImportInfo & info,
         vigra::importImage(info, dest);
         // integer image.. scale to 0 .. 1
         double scale = 1.0/vigra_ext::LUTTraits<SrcPixelType>::max();
-//        DestPixelType factor(scale);
-        vigra_ext::Multiply<double> f(scale);
         vigra::transformImage(dest.first, dest.first + vigra::Diff2D(info.width(), info.height()), dest.second,
             dest.first, dest.second,
             vigra::functor::Arg1()*vigra::functor::Param(scale));
     }
 }
 
-#if 0
-
-template <class SrcPixelType,
-          class DestIterator, class DestAccessor>
-void ImageCache::importAndConvertGrayImage(const ImageImportInfo & info,
-                                           vigra::pair<DestIterator, DestAccessor> dest,
-                                           wxString type)
-
-{
-    typedef typename DestAccessor::value_type DestPixelType;
-    typedef typename DestPixelType::value_type DestComponentType;
-    typedef SrcPixelType SrcComponentType;
-
-    // load image into temporary buffer.
-    BasicImage<SrcPixelType> tmp(info.width(), info.height());
-    vigra::importImage(info, destImage(tmp));
-
-    SrcComponentType min,max;
-
-    int range = wxConfigBase::Get()->Read(wxT("/ImageCache/Range"), HUGIN_IMGCACHE_RANGE);
-    if (range == 0) {
-        double t;
-        wxConfigBase::Get()->Read(wxT("/ImageCache/") + type + wxT("/min"), &t, double(GetRange<SrcComponentType>::min()));
-        min = SrcComponentType(t);
-        wxConfigBase::Get()->Read(wxT("/ImageCache/") + type + wxT("/max"), &t, double(GetRange<SrcComponentType>::max()));
-        max = SrcComponentType(t);
-    } else if (range == 1) {
-        vigra::FindMinMax<SrcComponentType> minmax;   // init functor
-        vigra::inspectImage(srcImageRange(tmp),
-                            minmax);
-        min = minmax.min;
-        max = minmax.max;
-    } else{
-        vigra_fail("Unknown image import range mode");
-    }
-
-    int mapping = wxConfigBase::Get()->Read(wxT("/ImageCache/Mapping"), HUGIN_IMGCACHE_MAPPING);
-    applyMapping(srcImageRange(tmp), dest, min, max, mapping);
-}
-
-template <class SrcPixelType,
-          class DestIterator, class DestAccessor>
-void ImageCache::importAndConvertGrayAlphaImage(const ImageImportInfo & info,
-                                                vigra::pair<DestIterator, DestAccessor> dest,
-                                                wxString type)
-{
-    typedef typename DestAccessor::value_type DestPixelType;
-    typedef typename DestPixelType::value_type DestComponentType;
-    typedef SrcPixelType SrcComponentType;
-
-    // load image into temporary buffer.
-    BasicImage<SrcPixelType> tmp(info.width(), info.height());
-    BImage mask(info.width(), info.height());
-    vigra::importImageAlpha(info, destImage(tmp), destImage(mask));
-
-    SrcComponentType min,max;
-
-    int range = wxConfigBase::Get()->Read(wxT("/ImageCache/Range"), HUGIN_IMGCACHE_RANGE);
-    if (range == 0) {
-        double t;
-        wxConfigBase::Get()->Read(wxT("/ImageCache/") + type + wxT("/min"), &t, double(GetRange<SrcComponentType>::min()));
-        min = SrcComponentType(t);
-        wxConfigBase::Get()->Read(wxT("/ImageCache/") + type + wxT("/max"), &t, double(GetRange<SrcComponentType>::max()));
-        max = SrcComponentType(t);
-    } else if (range == 1) {
-        vigra::FindMinMax<SrcComponentType> minmax;   // init functor
-        vigra::inspectImage(srcImageRange(tmp),
-                            minmax);
-        min = minmax.min;
-        max = minmax.max;
-    } else{
-        vigra_fail("Unknown image import range mode");
-    }
-
-    int mapping = wxConfigBase::Get()->Read(wxT("/ImageCache/Mapping"), HUGIN_IMGCACHE_MAPPING);
-    applyMapping(srcImageRange(tmp), dest, min, max, mapping);
-}
-
-#endif
-
 template <class SrcPixelType,
           class DestIterator, class DestAccessor,
           class MaskIterator, class MaskAccessor>
@@ -940,6 +741,68 @@ void ImageCache::postEvent(RequestPtr request, EntryPtr entry)
     }
 }
 
+void ImageCache::removeRequest(RequestPtr request)
+{
+    // Remove the no longer wanted requests from the queues.
+    // We need to check everything, as images can be loaded synchronously after
+    // an asynchronous request for it was made, and also something could have
+    // given up waiting (e.g. when the user switches images faster than they
+    // load).
+    // Take this opportunity to give out the signals, for the image just loaded
+    // and anything else we spot.
+    for (std::map<std::string, RequestPtr>::iterator it = m_smallRequests.begin();
+        it != m_smallRequests.end();)
+    {
+        std::map<std::string, RequestPtr>::iterator next_it = it;
+        ++next_it;
+        if (it->second.unique())
+        {
+            // Last copy of the request is in the list.
+            // Anything requesting it must have given up waiting.
+            m_smallRequests.erase(it);
+        }
+        else 
+        {
+            // remove all copies from request list
+            while (!it->second->ready.empty())
+            {
+                it->second->ready.erase(it->second->ready.begin());
+            };
+            m_smallRequests.erase(it);
+        }
+        it = next_it;
+    }
+    for (std::map<std::string, RequestPtr>::iterator it = m_requests.begin();
+        it != m_requests.end();)
+    {
+        std::map<std::string, RequestPtr>::iterator next_it = it;
+        ++next_it;
+        if (it->second.unique())
+        {
+            // The last copy of the request is in the list of requests.
+            // Anything that requested it must have given up waiting.
+            // Forget about it without loading.
+            m_requests.erase(it);
+        }
+        else 
+        {
+            // remove all parallel requests
+            while (!it->second->ready.empty())
+            {
+                it->second->ready.erase(it->second->ready.begin());
+            };
+            m_requests.erase(it);
+        }
+        it = next_it;
+    }
+    // If there are more images to load, start the thread again.
+    if (!(m_requests.empty() && m_smallRequests.empty()))
+    {
+        // Start a background thread to load another image.
+        spawnAsyncThread();
+    }
+}
+
 void ImageCache::spawnAsyncThread()
 {
     // Pick an image to load.
@@ -951,12 +814,8 @@ void ImageCache::spawnAsyncThread()
         {
             DEBUG_DEBUG("Not staring a thread to load an image, since no images are wanted.");
         } else {
-#ifdef USE_CXX11_THREAD
             std::thread thread(loadSafely, it->second, EntryPtr());
             thread.detach();
-#else
-            boost::thread(loadSafely, it->second, EntryPtr());
-#endif
         }
     } else {
         // got a small image request, check if its larger version has loaded.
@@ -966,20 +825,12 @@ void ImageCache::spawnAsyncThread()
         {
             // the larger one is needed to generate it.
             RequestPtr request(new Request(filename, false));
-#ifdef USE_CXX11_THREAD
             std::thread thread(loadSafely, request, EntryPtr());
             thread.detach();
-#else
-            boost::thread(loadSafely, request, EntryPtr());
-#endif
         } else {
             // we have the large image.
-#ifdef USE_CXX11_THREAD
             std::thread thread(loadSafely, it->second, large);
             thread.detach();
-#else
-            boost::thread(loadSafely, it->second, large);
-#endif
         }
     }
     // thread should be processing the image asynchronously now.
